from abc import abstractmethod


class BaseAggregator:
    @abstractmethod
    def __init__(self, need_dir_map=False, uid=None, working_dir=None, filenames=None, parse=True):
        self.filenames = filenames or []
        self.need_dir_map = need_dir_map
        self.working_dir = working_dir
        self.parse = parse
        self.uid = uid or self.__class__.__name__

    def filter(self, simulation):
        """
        Decide whether analyzer should process a simulation
        :param simulation: simulation object
        :return: Boolean whether simulation should be analyzed by this analyzer
        """
        return True

    def select_simulation_data(self, data, simulation):
        """
        In parallel for each simulation, consume raw data from filenames and emit selected data
        :param data: simulation data. Dictionary associating filename with content
        :param simulation: object representing the simulation for which the data is passed
        :return: selected data for the given simulation
        """
        return {}
